////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BIASED_LIGHTMAP_H
#define BIASED_LIGHTMAP_H

#include "lightmap.h"

#include <vector>


class CBiasedLightMap : public CLightMap
{
public:
  CBiasedLightMap();
  virtual ~CBiasedLightMap(void);

  virtual bool    LoadMap( const std::string& FileName );

protected:

	// sample the color + the direction of the light ray
	virtual Color   Sample_f( GeometryEngine* pEngine, const Ray& Out, int SampleNr, Vector& LightDir );

  virtual void    CalculateBiasingDistributions();
  virtual void    SampleDirectionUV( const Vector& Normal, Random& Sampler, unsigned int SampleNr, Vector& Dir, float& UTexel, float& VTexel );

	//////////////////////////////////////////////////////////////////////////////

	template<typename TVal, typename TIndex>
	TIndex BinSearchIndex( TVal* pData, const TVal& SearchVal, TIndex& MinIndex, TIndex& MaxIndex )

	//////////////////////////////////////////////////////////////////////////////
	{
		TIndex SplitIndex;
		//  assert( MinIndex >= 0 && MaxIndex < ArraySize<TVal*>( pData) );

		// check for over- or underflow of the search range
		if( SearchVal < pData[MinIndex] || SearchVal > pData[MaxIndex] ) {
			return -1;
		}
		else if( SearchVal == pData[MaxIndex] )
			return MaxIndex;

		// Could we narrow our search down to one bin?
		if( MaxIndex == MinIndex + 1 && SearchVal >= pData[MinIndex] && SearchVal < pData[MaxIndex] ) {
			return MinIndex;
		}
		// Otherwise we try to create the binary search tree
		else {
			SplitIndex = MinIndex + (TIndex)((MaxIndex - MinIndex) / 2.0);
			if( SearchVal >= pData[MinIndex] && SearchVal < pData[SplitIndex] )
				return BinSearchIndex( pData, SearchVal, MinIndex, SplitIndex );
			else
				return BinSearchIndex( pData, SearchVal, SplitIndex, MaxIndex );
		}

		return -1;
	}


private:
  Color           LookUpColor( float UTexel, float VTexel );

protected:

  std::vector<std::vector<double> > m_ColumnCDF;      // cumulative distribution function for each column
  std::vector<double>               m_IntegralRowCDF; // CDF that integrates the CDF's of all columns
};


#endif