////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matclearglass.h"

class CBxDFPhong;
class CFresnelPreDefDiel;
class CBxDFSpecReflection;
class CBxDFSpecTransmission;

CMatClearGlass::CMatClearGlass( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_TRANSMISSION | CBxDFBase::BSDF_REFLECTION ), false /* false = no MC sampling required */ )
{
  m_pBxDFDiffuse = new CBxDFPhong;
  m_pBxDFTransmission = new CBxDFSpecTransmission( Color( 0.92, 0.92, 1.0)) ; // specular transmission with slight taint of blue
}

CMatClearGlass::~CMatClearGlass(void)
{
  delete m_pBxDFDiffuse;
  delete m_pBxDFTransmission;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatClearGlass::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatClearGlass::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  double DotInHalf;

  if( Type == CBxDFBase::BSDF_REFLECTION ) {      // sample Fresnel scaled reflection

    Vector HalfVec(-Out.dir() + In.dir() );
    HalfVec.normalize();
    DotInHalf = In.dir().dot( HalfVec );

    double ReflectedContribution = m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);
    return CFresnelHelpers::SchlickFresnel( ReflectedContribution, DotInHalf );
  }
  else if( Type == CBxDFBase::BSDF_TRANSMISSION ) // sample transmission
    return m_pBxDFTransmission->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade );
}
// vim:ts=2
