////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matglass.h"

CMatGlass::CMatGlass( CBaseIntegrator* pInt, float RefractionIndex ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_TRANSMISSION | CBxDFBase::BSDF_REFLECTION ), false /* false = no MC sampling required */ )
{
  m_pBxDFDiffuse = new CBxDFPhong;
  m_pBxDFTransmission = new CBxDFSpecTransmission; // specular transmission
  m_pBxDFFresnelDiel = new CFresnelPreDefDiel( 1.0, RefractionIndex );
  m_pBxDFReflection = new CBxDFSpecReflection( m_pBxDFFresnelDiel );
}

CMatGlass::~CMatGlass(void)
{
  delete  m_pBxDFDiffuse;      // lambertian diffuse reflector
  delete  m_pBxDFTransmission; // specular transmission
  delete  m_pBxDFReflection;
  delete  m_pBxDFFresnelDiel;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatGlass::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );
  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatGlass::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION )
    return m_pBxDFReflection->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);
  else if( Type == CBxDFBase::BSDF_TRANSMISSION )
    return m_pBxDFTransmission->Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade);

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade );
}
// vim:ts=2
