////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MAT_GLASS_H
#define MAT_GLASS_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "bxdffresnel.h"
#include "bxdfmicrofacet.h"
#include "bxdfphong.h"
#include "blinnmicrodist.h"
#include "bxdfspecrefl.h"
#include "bxdfspectransmission.h"
#include "bxdftranslucent.h"
#include "baseintegrator.h"


class CBxDFPhong;
class CFresnelPreDefDiel;
class CBxDFSpecReflection;
class CBxDFSpecTransmission;

class CMatGlass : public CPhysMatInterface
{
public:
  CMatGlass( CBaseIntegrator* pInt, float RefractionIndex );
  virtual ~CMatGlass(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0 );

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );

protected:
  CBxDFPhong*             m_pBxDFDiffuse;      // lambertian diffuse reflector
  CBxDFSpecTransmission*  m_pBxDFTransmission; // specular transmission
  CBxDFSpecReflection*    m_pBxDFReflection;   // specular reflection
  CFresnelPreDefDiel*     m_pBxDFFresnelDiel;
};


#endif
// vim:ts=2
