////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MAT_SHINY_METAL_H
#define MAT_SHINY_METAL_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "baseintegrator.h"
#include "bxdffresnel.h"
#include "bxdfmicrofacet.h"
#include "bxdfphong.h"
#include "blinnmicrodist.h"
#include "bxdfspecrefl.h"
#include "bxdfspectransmission.h"

class CBxDFPhong;
class CFresnelPreDefConductor;
class CBxDFSpecReflection;
class CBxDFSpecTransmission;

class CMatShinyMetal : public CPhysMatInterface
{
public:
  explicit CMatShinyMetal( CBaseIntegrator* pInt, const float cRoughness = 1.0 );
  virtual ~CMatShinyMetal(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0 );

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );

protected:
  CBxDFPhong*               m_pBxDFDiffuse;      // microfacet diffuse reflector
  CBxDFSpecReflection*      m_pBxDFSpecular;     // specular reflection
  CFresnelPreDefConductor*  m_pFresnelReflection;
  CFresnelPreDefConductor*  m_pFresnelSpecular;
  CBlinnMicroDist*          m_pBlinnMicroDist;
  CBxDFMicrofacet*          m_pDiffuseMicroFacets;
};

#endif
// vim:ts=2
