/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 *
 * Date:	16-Feb-2016
 */
#ifndef __GEOMETRY_LINE_H
#define __GEOMETRY_LINE_H

#include "math/vector.h"

class Point;

/** Calculate the shortest distance between a point and a line
 * @param point to find distance
 * @param lineStart start point of line
 * @param lineEnd end point of line
 * @param distance if not NULL return the distance
 * @return TRUE is the shortest distance is in the segment between the
 *	lineStart lineEnd
 */
bool pointLineDistance( const Point& point,
		const Point& start, const Point& end,
		double* distance=nullptr);

/** @return the minimum distance between two lines */
double lineLineDistance(const Point& p1, const Vector& z1,
			const Point& p2, const Vector& z2);

/** Calculate the line segment PaPb that is the shortest route between
 * two lines P1P2 and P3P4. Calculate also the values of mua and mub where
 *    Pa = P1 + mua (P2 - P1)
 *    Pb = P3 + mub (P4 - P3)
 * @param p1,p2 points defining the first line segment
 * @param p3,p4 points defining the second line segment
 * @param pa,pb if not NULL return the position of intersection on both lines
 * @param mua,mub return the distance from p1,p3 respectively
 * @param eps the required accuracy of the operation
 * @return false if no solution exists.
 */
bool lineLineIntersect(
		const Point& p1, const Point& p2,
		const Point& p3, const Point& p4,
		Point*  pa,   Point*  pb,
		double* mua,  double* mub,
		const double eps=Vector::_epsilon);
#endif
