/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#include "quabody.h"

// ================================= GQUABody ==================================
// FIXME position, get, Mesh...
void GQUABody::set(double *what)
{
	resetBBox();
	round0(what, nwhats(), SMALL1);
	q = Quad(what[0], what[1], what[2], what[3], what[4],
		what[5], what[6], what[7], what[8], what[9]);

	Matrix3 M;
	q.matrix3(&M);
	Point v(-q.Cx/2.0, -q.Cy/2.0, -q.Cz/2.0);
	M.inverse();
	P = M * v;
} // set

/* --- get --- */
int GQUABody::get(double *what) const
{
	what[0] = q.Cxx;
	what[1] = q.Cyy;
	what[2] = q.Czz;
	what[3] = q.Cxy;
	what[4] = q.Cxz;
	what[5] = q.Cyz;
	what[6] = q.Cx;
	what[7] = q.Cy;
	what[8] = q.Cz;
	what[9] = q.C;
	return 10;
} // get

/** position
 * @param x,y,z	new position to set
 */
void GQUABody::position(const Point& r)
{
	GBody::position(r);

	Point tr = _hasMatrix? _invMatrix*r : r;
	tr -= SP;
	q = sq;
	q.translate(-tr);
} // position

/** save */
void GQUABody::save()
{
	GBody::save();
	sq = q;
} // save

/** restore */
void GQUABody::restore()
{
	q = sq;
	GBody::restore();
} // restore

/** move body item to location
 * @param item	item to move as returned by closest
 * @param r	position vector
 * @param w	vertical axis of the viewport
 */
void GQUABody::move(int item, const Point& r, const Vector& w)
{
	resetBBox();
	if (item==1) {
		Point tr = _hasMatrix? _invMatrix*r : r;
		tr -= SP;

		double a = sq.Cxx*Sq(tr.x)
			 + sq.Cyy*Sq(tr.y)
			 + sq.Czz*Sq(tr.z);

		q = sq;
		if (!Eq0(a,TOOSMALL)) {
			double b = sq.Cxy*tr.x*tr.y
				 + sq.Cxz*tr.x*tr.z
				 + sq.Cyz*tr.y*tr.z
				 + sq.Cx*tr.x
				 + sq.Cy*tr.y
				 + sq.Cz*tr.z
				 + sq.C;
			double t = -b/a;
			q.Cxx *= t;
			q.Cyy *= t;
			q.Czz *= t;
		}
	} else
		GBody::move(item, r, w);
} // move

/** rotate body around an axis */
void GQUABody::rotate(const double angle, const Vector& axis)
{
	Matrix4	Rot, RotInv;
	Rot.rotate(-angle, axis);
	RotInv.rotate(angle, axis);

	q = sq;
	q.translate(SP);
	q.transform(Rot, RotInv);
	q.translate(-SP);
	resetBBox();
} // rotate

/** @return bounding box of body */
void GQUABody::updateBBox() const
{
	_bbox = Q(0).bbox();
} // bbox

#ifdef OBBOX
/** @return oriented bounding box of body */
void GQUABody::updateOBB(bool in) const
{
	OBBox& bb = _obbox[(int)in];
	if (!in)
		bb = OBBox(Q(0).obbox());
} // updateOBB
#endif
