/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include <fstream>
#include <iostream>

#include "os.h"
#include "nox.h"
#include "tool/timer.h"
#include "math/random.h"
#include "physics/particle.h"

/** main */
int main(int ac, char *av[])
{
	if (ac!=3) {
		cerr << "syntax: " << av[0] << " <file.nox> <events>" << endl;
		return 1;
	}
	fpetrap();
	NoxGeometry world(av[1]);

	GeometryEngine* engine = world.viewer.kernel().engine();

	Random random(1234567890);
	int nevents = atoi(av[2]);

	Timer timer;
	timer.start();

#if 0
	for (int i=0; i<nevents; i++) {
		double x, y, z;
		//random.normal(&x,&y);
		//random.sincos(&x,&y);
		//random.disc(2.0,&x,&y);
		random.sphere(2.0,&x,&y,&z);
		cout << x << " " << y << " " << z << endl;
	}
#endif

	for (int i=0; i<nevents; i++) {
		Caloron particle;
		Vector pos = random.sphere(2.0);
		cout << pos.x << " " << pos.y << " " << pos.z << endl;
		VZone* zone = NULL;

		while (true) {
			Vector dir = random.vector();		// random direction
			double step = random.normal();		// random step

			engine->incBodyCheckId();
			zone = engine->whereRay(pos, dir, SMALL3D, zone);

			particle.init();
			RaySegment segment(pos, dir, zone);
			segment.tmax = step;
			particle.push(segment);

			// find next intersection
			if (engine->intersectRay(&particle, true)) {
				pos = particle.hit();
				cout << pos.x << " " << pos.y << " " << pos.z << endl;
			} else {
				break;
			}
		}
		cout << endl << endl;
	}

	timer.stop();
	cout << "# Timer1:" << timer << endl;
} // main
