/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	31-Mar-2010
 */

#include <memory>    // For std::unique_ptr

#include "bstring.h"

#include <string>
#include <ostream>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

/** string hash algorithm from http://www.cse.yorku.ca/~oz/hash.html
 djb2
 this algorithm (k=33) was first reported by Dan Bernstein many years ago in
 comp.lang.c. another version of this algorithm (now favored by Bernstein) uses
 xor: hash(i) = hash(i - 1) * 33 ^ str[i]; the magic of number 33 (why it works
 better than many other constants, prime or not) has never been adequately
 explained.
*/
dword hash_djb2(const char* str)
{
	dword hash = 5381;
	int c;

	while ((c=*(const byte*)str++))
		hash += (hash << 5) + c; /* hash * 33 + c */

	return hash;
} // hash_djb2

/** create a name000number format fitting inside length
 * with a name and a number with minimum digits
 * @param output	output string of size
 * @param size		size of output string
 * @param name		name as a prefix
 * @param number	number as a suffix
 * @param digits	minimum number of digits to append
 * @return output
 */
char* nameNumber(char* output, int size, const char* name, int number, int digits)
{
	strncpy(output, name, size);
	output[size-1] = '\0';
	char num[16];
	sprintf(num,"%0*d",digits,number);

	int slen = strlen(output);
	int nlen = strlen(num);

	if (slen + nlen < size-1)
		strcpy(output+slen, num);
	else {
		int pos = size-1-nlen;
		if (pos<=0)
			strcpy(output+1, num+1-pos);
		else
			strcpy(output+pos, num);
	}
	return output;
} // nameNumber

/** string format like printf() for strings
 * from: https://stackoverflow.com/questions/2342162/stdstring-formatting-like-sprintf
 */
std::string sformat(const char* fmt, ...)
{
	std::unique_ptr<char[]> formatted;
	int n = (int)strlen(fmt) * 2;	// Reserve two times as much as the length of the fmt_str
	while (true) {
		// Wrap the plain char array into the unique_ptr
		formatted.reset(new char[n]);
		strcpy(&formatted[0], fmt);
		va_list ap;
		va_start(ap, fmt);
		int final_n = vsnprintf(&formatted[0], n, fmt, ap);
		va_end(ap);
		if (final_n < 0 || final_n >= n)
			n += abs(final_n - n + 1);
		else
			break;
	}
	return std::string(formatted.get());
} // sformat
