/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	31-Mar-2010
 */

#ifndef __TOOL_BSTRING_H
#define __TOOL_BSTRING_H

#include <string>
#include <algorithm>

#include "os.h"

/* --------------- NON UNICODE ----------------- */
#if !defined(TCHAR)
	#if defined(UNICODE)
		#define	TCHAR		wchar_t
		#define	LPTSTR		wchar_t*
		#define	LPCTSTR		const wchar_t*
	#else
		#define	TCHAR		char
		#define	LPTSTR		char*
		#define	LPCTSTR		const char*
	#endif
	#ifndef TEXT
		#define	TEXT(x)		(x)
	#endif
#endif

/* ----------------- Strings --------------------- */
#if defined(UNIX) ||defined(unix) || defined(__unix__) || defined(__unix)
	#if defined(UNICODE)
		#define _tcscmp		wcscmp
		#define _tcsncmp		wcsncmp
		#define _tcscpy		wcscpy
		#define _tcslen		wcslen
	#else
		#define _tcscmp		strcmp
		#define _tcsncmp		strncmp
		#define _tcscpy		strcpy
		#define _tcslen		strlen
		#define _ttoi		atoi
	#endif
#endif

#if defined(__BORLANDC__)	/* For the HUGE/LARGE memory model */
	#define	STRCPY		_fstrcpy
	#define	STRCMP		_fstrcmp
	#define	STRNCMP		_fstrncmp
	#define	STRCAT		_fstrcat
	#define	STRCHR		_fstrchr
	#define	STRLEN		_fstrlen
	#define	STRSTR		_fstrstr
	#define	STRNCPY		_fstrncpy
	#define	SPRINTF		sprintf
	#define	MKTEMP		mktemp
#elif defined(USE_UNICODE)
	#define	STRCPY		wcscpy
	#define	STRCMP		wcscmp
	#define	STRNCMP		wcsncmp
	#define	STRNCPY		wcsncpy
	#define	STRCAT		wcscat
	#define	STRCHR		wcschr
	#define	STRLEN		wcslen
	#define	STRDUP		wcsdup
	#define	SPRINTF		swprintf
#elif defined(HAS_STRING)
	#define	STRCPY		strcpy
	#define	STRCMP		strcmp
	#define	STRNCMP		strncmp
	#define	STRNCPY		strncpy
	#define	STRCAT		strcat
	#define	STRCHR		strchr
	#define	STRLEN		strlen
	#define	STRSTR		strstr
	#define	STRUPR		strupr
	#define	STRDUP		strdup
	#define	SPRINTF		sprintf
	#define	MKTEMP		mktemp
#else
	#define	STRCPY		Bstrcpy
	#define	STRCMP		Bstrcmp
	#define	STRNCMP		Bstrncmp
	#define	STRCAT		Bstrcat
	#define	STRCHR		Bstrchr
	#define	STRLEN		Bstrlen
	#define	STRSTR		Bstrstr
	#define	STRUPR		Bstrupr
	#define	SPRINTF		Bsprintf
#endif

/* ----------------- Ctype ------------------------- */
#if defined(USE_UNICODE)
	#define	TOUPPER		towupper
	#define	TOLOWER		towlower
	#define	ISDIGIT		iswdigit
	#define	ISPRINT		iswprint
#elif defined(HAS_CTYPE)
	#define	TOUPPER		toupper
	#define	TOLOWER		tolower
	#define	ISSPACE		isspace
	#define	ISDIGIT		isdigit
	#define	ISPRINT		isprint
	#define	ISXDIGIT	isxdigit
	#define	ISALPHA		isalpha
#else
	#define	ISSPACE		Bisspace
	#define	ISDIGIT		Bisdigit
	#define	ISXDIGIT	Bisxdigit
	#define	ISALPHA		Bisalpha
#endif

dword	hash_djb2(const char *str);
//dword	hash_djb2(const std::string& str)	{ return hash_djb2(str.c_str()); }
char*	nameNumber(char* output, int size, const char* name, int number, int digits=2);
std::string sformat(const char* fmt, ...);

/** to_upper */
template<typename T>
inline T to_upper(T text) {
	std::transform(std::begin(text), std::end(text), std::begin(text), ::toupper);
	return text;
}

/** to_lower */
template<typename T>
inline T to_lower(T text) {
	std::transform(std::begin(text), std::end(text), std::begin(text), ::tolower);
	return text;
}

/** strip */
template<typename T>
inline T strip(T const& text)
{
	auto first {text.find_first_not_of(' ')};
	auto last  {text.find_last_not_of(' ')};
	return text.substr(first, (last - first + 1));
}

/** lstrip */
template<typename T>
inline T lstrip(T const& text)
{
	auto first {text.find_first_not_of(' ')};
	return text.substr(first, text.size() - first);
}

/** rstrip */
template<typename T>
inline T rstrip(T const& text)
{
	auto last{ text.find_last_not_of(' ') };
	return text.substr(0, last + 1);
}

/** trim from start (in place) */
inline void ltrim(std::string& str)
{
	str.erase(str.begin(), std::find_if(str.begin(), str.end(),
		[](int ch) {return !std::isspace(ch);}));
} // ltrim

/** trim from end (in place) */
inline void rtrim(std::string& str)
{
	str.erase(std::find_if(str.rbegin(), str.rend(),
		[](int ch) { return !std::isspace(ch); }).base(),
		str.end());
} // rtrim

/** trim from both ends (in place) */
inline void trim(std::string& str)
{
	ltrim(str);
	rtrim(str);
} // trim

//To remove characters from a string, use std::remove_if() and std::basic_string::erase():
//template<typename CharT>
//inline T remove(T text, T const ch)
//{
//	auto start = std::remove_if(
//			std::begin(text), std::end(text),
//			[=](CharT const c) {return c ==  ch; });
//	text.erase(start, std::end(text));
//	return text;
//}

// To split a string based on a specified delimiter, use std::getline() to read
// from an std::basic_stringstream initialized with the content of the string.
// The tokens extracted from the stream are pushed into a vector of strings:
//template<typename T>
//inline std::vector<T> split(T text, char const delimiter)
//{
//	auto sstr = tstringstream<CharT>{ text };
//	auto tokens = std::vector<T>{};
//	auto token = T{};
//	while (std::getline(sstr, token, delimiter)) {
//		if (!token.empty()) tokens.push_back(token);
//	}
//	return tokens;
//}

#endif
